#include <iostream>
using namespace std;

#include <math.h>

#include "main.h"


void
free_spline_workspaces(SplineInfo& spinfo) 
{

  if ( NULL != spinfo.int_workspace )
  {
    free( spinfo.int_workspace ); 
    spinfo.int_workspace = NULL;
  }

  if ( NULL != spinfo.dbl_workspace )
  {
    free( spinfo.dbl_workspace ); 
    spinfo.dbl_workspace = NULL;
  }
}

void
alloc_spline_workspaces(SplineInfo& spinfo )
{
  spline_coef& s_scoef = spinfo.scoef;

  int mxorder = (s_scoef.theta_order > s_scoef.z_order) 
              ? s_scoef.theta_order 
              : s_scoef.z_order;

  spinfo.liwmax = 1 + s_scoef.num_theta_knots + s_scoef.theta_order;

  spinfo.int_workspace = (integer*) malloc( spinfo.liwmax * sizeof(integer) );

  if (! spinfo.int_workspace )
  {
    cerr << "unable to allocate integer workspace" << endl;
    exit(EXIT_FAILURE);
  }

  
  spinfo.lwmax  = s_scoef.theta_order * 2 
                 + 2 * s_scoef.num_theta_knots + s_scoef.z_order + 2 * mxorder;

  spinfo.dbl_workspace = (double*) malloc( spinfo.lwmax * sizeof(double) );

  if (! spinfo.dbl_workspace )
  {
    cerr << "unable to allocate double workspace" << endl;
    exit(EXIT_FAILURE);
  }
}


/*-----------------------------------------------------------------------*/

BsplineErr
dasl_eval( 
  doublereal theta, 
  doublereal zn, 
  SplineInfo& spinfo, 
  double* s
)
{
  spline_coef& scoef = spinfo.scoef;

  // handle conversions from size_t to integer
  integer  z_order     = scoef.z_order;
  integer  theta_order = scoef.theta_order;
  integer  qz          = scoef.qz;
  integer  num_theta_knots = scoef.num_theta_knots;

  integer     ifail     = 0;
  integer     rc        = 0;
  integer     one       = 1;


  if ( theta < 0.0        ) { theta += M_2_PI; }
  if ( theta > scoef.tmax ) { theta = fmod( theta, M_2_PI ); }

  /*
   * evaluate spline deformation
   */
  B2VPRE_F77( &z_order, &qz,
	      &scoef.zmin, &scoef.zmax, scoef.z_knots,
	      &theta_order, &num_theta_knots,
	      &scoef.tmin, &scoef.tmax, scoef.t_knots,
	      scoef.c, &qz, &one, &zn, &one, &theta,
	      s, &one,
	      spinfo.dbl_workspace, &spinfo.lwmax,
	      spinfo.int_workspace, &spinfo.liwmax, &ifail );
  rc += ifail;

  return rc ? Bspline_OutOfRange : Bspline_Success;
}


BsplineErr
dasl_eval_derivs( 
  doublereal theta, 
  doublereal zn, 
  SplineInfo& spinfo, 
  BsplineResult* bspres
)
{
  spline_coef& scoef = spinfo.scoef;

  // handle conversions from size_t to integer
  integer  z_order     = scoef.z_order;
  integer  theta_order = scoef.theta_order;
  integer  qz          = scoef.qz;
  integer  num_theta_knots = scoef.num_theta_knots;

  integer     qz_1      = scoef.qz - 1;
  integer     z_order_1 = scoef.z_order - 1;
  integer     t_order_1 = scoef.theta_order - 1;
  integer     ifail     = 0;
  integer     rc        = 0;
  integer     one       = 1;

  doublereal dsdz, dsdt;

  if ( theta < 0.0        ) { theta += M_2_PI; }
  if ( theta > scoef.tmax ) { theta = fmod( theta, M_2_PI ); }

  /* 
   * evaluate spline deformation z-derivative
   */
  B2VPRE_F77( &z_order_1, &qz_1,
	      &scoef.zmin, &scoef.zmax, scoef.z_knots,
	      &theta_order, &num_theta_knots,
	      &scoef.tmin, &scoef.tmax, scoef.t_knots,
	      scoef.cdifz, &qz_1, &one, &zn, &one, &theta,
	      &dsdz, &one,
	      spinfo.dbl_workspace, &spinfo.lwmax,
	      spinfo.int_workspace, &spinfo.liwmax, &ifail );
  rc += ifail;

  ifail = 0;

  /* 
   * evaluate spline deformation theta-derivative
   */
  B2VPRE_F77( &z_order, &qz,
	      &scoef.zmin, &scoef.zmax, scoef.z_knots,
	      &t_order_1, &num_theta_knots,
	      &scoef.tmin, &scoef.tmax, scoef.t_knots,
	      scoef.cdift, &qz, &one, &zn, &one, &theta,
	      &dsdt, &one,
	      spinfo.dbl_workspace, &spinfo.lwmax,
	      spinfo.int_workspace, &spinfo.liwmax, &ifail );
  rc += ifail;

  bspres->dsdz = dsdz;
  bspres->dsdt = dsdt;

  return rc ? Bspline_OutOfRange : Bspline_Success;
}
